#!/bin/sh
# --- tcl/tk-GUI for monte --- (c) Joerg Schulenburg
#  start: wish monte.tcl
#
#  tested on: tk8.0 (linux,win95)
#
# ToDo:
#  - better using wishx with extended Tcl? (for pipe etc.)
#  - Reading from stderr of child???
#
# this is a multiline comment in wish, but not in sh \
exec wish -f $0 $@

set ver 0.1.2

#
wm title . "Monte Tcl frontend $ver"

set kT 1.30
set field 0
set steps 100
set imgviewer "xli -zoom 400"
set run "./monte -b"
set file "monte.pgm"
if  $argc {
 puts " ignore $argc arguments: $argv"
}

frame .mbar -borderwidth 1 -relief raised
pack .mbar -fill x

menubutton .mbar.file -text "File" -menu .mbar.file.m
pack .mbar.file -side left
menu .mbar.file.m
.mbar.file.m add command -label "Clear output" -command {$log delete 0.0 end}
.mbar.file.m add command -label "Exit" -command exit


menubutton .mbar.options -text "Options" -menu .mbar.options.m
pack .mbar.options -side left
menu .mbar.options.m
.mbar.options.m add command -label "set Options" -command Option

menubutton .mbar.help -text "Help" -menu .mbar.help.m
pack .mbar.help -side right
menu .mbar.help.m
.mbar.help.m add command -label "About" -command aboutBox
# .mbar.help.m add command -label "Version"

# --------------------- action frame -----------------------
frame .abar -borderwidth 1 -relief sunken
pack .abar -fill x

set but [button .abar.go -text "Run it" -command Run]
#button .abar.go -text "Run it" -command { Run }
pack .abar.go -side left

set f .abar
set w .abar
button .abar.show -text "Show" -command "Show"
pack .abar.show -side right

# ------------------ canvas, text ? -----------------
canvas .pad -background gray
set imh 0
# set imh [image create photo -file $file]
.pad create image 0 0
# -image $imh
pack .pad -side top -fill both -expand true

frame .t
#text .t
#pack .t
##.t insert end "history:"
##.t configure -state disabled

set log [text .t.log -width 80 -height 20 \
  -borderwidth 2 -relief raised -setgrid true\
  -yscrollcommand {.t.scroll set}]
set log2 [text .log2 -width 80 -height 2 \
  -borderwidth 2 -relief raised -setgrid true]
scrollbar .t.scroll -command {.t.log yview}
pack .t.scroll -side right -fill y
pack .t.log -side left -fill both -expand true
pack .t -side top -fill both -expand true
pack .log2 -side top -fill x -expand true
#####################################################

bind .abar <Return> Run
bind .abar <Control-c> Stop
focus .abar


# --------------------- Show Picture ---------------------
proc Show {} {
  global file imgviewer
  eval exec $imgviewer $file &
}

# see /usr/lib/tk8.0/demos/check.tcl
proc Option {} {
  global kT field imgviewer run
  set w .woption
  catch {destroy $w}
  toplevel $w
  wm title $w "Monte Options"

set ww $w

 label $ww.msg -wraplength 4i -justify left -text "Here you can set several options."
 pack $ww.msg -side top

 frame $w.buttons
 pack $w.buttons -side bottom -fill x -pady 2m
 button $w.buttons.ok   -text "Ok" -command "destroy $w"
 button $w.buttons.vars -text "See Variables" -command "showVars"
 pack $w.buttons.vars $w.buttons.ok -side left -expand 1

 frame $ww.sep1 -relief ridge -bd 1 -height 2
 pack $ww.sep1 -side top -fill x
#  -expand no
 
 frame $ww.scale1
 pack $ww.scale1 -side top -fill x
 label $ww.scale1.label -text "kT:" -width 8
 scale $ww.scale1.scale -orient horizontal -length 200 -from 0 -to 4 \
        -variable kT -command SetKT -resolution 0.001
 pack $ww.scale1.label $ww.scale1.scale -side left
# checkbutton $ww.scale1.check -text "auto" -variable autos -onvalue  1 -offvalue  0
# pack $ww.scale1.check -side left
 
 frame $ww.scale2
 pack $ww.scale2 -side top -fill x
 label $ww.scale2.label -text "field:" -width 8
 scale $ww.scale2.scale -orient horizontal -length 200 -from 0 -to 4 \
        -variable field -command SetField -resolution 0.001
 pack $ww.scale2.label $ww.scale2.scale -side left
 
 frame $ww.scale3
 pack $ww.scale3 -side top -fill x
 label $ww.scale3.label -text "steps:" -width 8
 scale $ww.scale3.scale -orient horizontal -length 200 -from 0 -to 2000 \
        -variable steps -resolution 1
 pack $ww.scale3.label $ww.scale3.scale -side left
 
 frame $ww.sep2 -relief ridge -bd 1 -height 2
 pack $ww.sep2 -side top -fill x
#  -expand no

 # place the checkbuttons in two columns
 frame $ww.cb
 pack $ww.cb -side top -fill x
 frame $ww.cb.cb1 -relief ridge -bd 1 -height 2
 pack $ww.cb.cb1 -side left -fill y
 frame $ww.cb.sep3 -relief ridge -bd 1 -height 2
 pack $ww.cb.sep3 -side left -fill y
 frame $ww.cb.cb2 -relief ridge -bd 1 -height 2
 pack $ww.cb.cb2 -side left -fill y

set ww $w.cb.cb1

 set m1 1
 label $ww.msg1 -text "Gitter initialisieren:"
 pack $ww.msg1 -side top
 radiobutton $ww.b1 -text "random bonds"         -variable m1 -value 0 -command { puts $fid bz }
 radiobutton $ww.b2 -text "ferromagnet Tc=2.269" -variable m1 -value 1 -command { puts $fid bf }
 radiobutton $ww.b3 -text "antiferromagnet"      -variable m1 -value 2 -command { puts $fid ba }
 radiobutton $ww.b4 -text "frustrated plaketts"  -variable m1 -value 3 -command { puts $fid bp }
 pack $ww.b1 $ww.b2 $ww.b3 $ww.b4 -side top -pady 2 -anchor w


set ww $w.cb.cb2

 label $ww.msg2 -text "Misc:"
 pack $ww.msg2 -side top
 button $ww.c1 -text "set random state"  -command { puts $fid zz }
 button $ww.c2 -text "set srandom 1   "  -command { puts $fid z1 }
 button $ww.c3 -text "set OBC         "  -command { puts $fid p }
 pack $ww.c1 $ww.c2 $ww.c3 -side top -pady 2 -anchor w

set ww $w

 frame $ww.sep4 -relief ridge -bd 1 -height 2
 pack $ww.sep4 -side top -fill x

set ww $w.l1
 frame $ww
 pack $ww -side top -fill x

 label $ww.d1 -text "Image Viewer:" -width 16
 entry $ww.d2 -textvariable imgviewer
 pack $ww.d1 -side left -pady 2 -padx 2 -anchor w
 pack $ww.d2 -side left -pady 2 -padx 2 -fill x -expand 1 -anchor w

set ww $w.l2
 frame $ww
 pack $ww -side top -fill x

 label $ww.e1 -text "EXEC:" -width 16
 entry $ww.e2 -textvariable run
 pack $ww.e1 -side left -pady 2 -padx 2 -anchor w
 pack $ww.e2 -side left -pady 2 -padx 2 -fill x -expand 1 -anchor w

set ww $w.l3
 frame $ww
 pack $ww -side top -fill x

}

# ----------------------------- showVars -----------------------------
proc showVars {} {
  global log file run
  $log insert end "run = $run file = $file\n"
  update
}

    
# ----------------------------- saveLog -----------------------------
proc SaveText { txtfile } {
  global log
  if [catch {open $txtfile w} out] {
    puts "open $txtfile failed"
    return
  }
  puts $out [$log get 0.0 end]
  close $out
}


# --- start backend ----    
$log insert end $run\n
$log see end
set fid [open "| $run" "r+"]
fconfigure $fid -buffering line
# monte must output with "...\n" and flush!
fileevent $fid readable Log
puts $fid "zz"
flush $fid
puts $fid "bf"
flush $fid
puts $fid "t$kT"
flush $fid


# ----------------------------- RUN -----------------------------
proc Run {} {
  global fid but log steps
  puts $fid "m$steps"
  flush $fid
  # $log insert end "m$steps\n"
  $but config -text "Stop !" -command Stop
  update
}

proc Log {} {
  global fid log log2 but imh file
  if [eof $fid] { Stop } else {
    gets $fid line
    if { [string first "E" $line] >= 0 } {
     if { $imh != 0 } {
       image delete $imh
     }
     # how to zoom?
     set imh [image create photo -file $file]
     .pad create image 100 100 -image $imh
     # puts "$line"
     $log2 delete 0.0 end
     $log2 insert end $line\n
     $log2 see end
    }
    if { [string first "E=" $line] >= 0 } {
      $log insert end $line\n
      $log see end
      Stop
    }
    # I want to update image here!
    update idletask
  }
}

proc Stop {} {
  global fid but
  $but config -text "Run it" -command Run
  update
}


proc Quit {} {
  global fid
  catch {close $fid}
}

proc SetField { field } {
  global fid
  puts $fid "f$field"
}

proc SetKT { kt } {
  global fid
  puts $fid "t$kt"
}

# aboutBox --
#
#       Pops up a message box with an "about" message
#
proc aboutBox {} {
  global ver montever
     tk_messageBox -icon info -type ok -title "About" -message \
    "Tcl/Tk front end\nto Monte $montever\n\nversion $ver (c) 2002 JS\njschulen@gmx.de"
}

